@@ -29,7 +29,6 @@
 #include <math.h>
 #include <string.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <locale.h>
 
 #include <libart_lgpl/art_affine.h>
@@ -40,6 +39,7 @@
 #include <libgnomeprint/gnome-print-multipage.h>
 #include <libgnomeprint/gnome-print-job-private.h>
 #include <libgnomeprint/gnome-print-config-private.h>
+#include <libgnomeprint/gnome-print-i18n.h>
 
 typedef struct _JOBPrivate JOBPrivate;
 
@@ -68,7 +68,7 @@ struct _JOBPrivate {
 enum
 {
   PROP_0,
-  PROP_CONFIG,
+  PROP_CONFIG
 };
  
 
@@ -81,7 +81,6 @@ static void gnome_print_job_finalize (GObject *object);
 static void job_update_layout_data (GnomePrintJob *job);
 static void job_parse_config_data (GnomePrintJob *job);
 static void job_clear_config_data (GnomePrintJob *job);
-static gboolean job_parse_transform (guchar *str, gdouble *transform);
 
 static void gnome_print_job_get_property (GObject      *object,
                                           guint         prop_id,
@@ -127,8 +126,8 @@ gnome_print_job_class_init (GnomePrintJobClass *klass)
 	g_object_class_install_property (object_class,
                                    PROP_CONFIG,
                                    g_param_spec_object ("config",
-                                                        "Job Configuration",
-                                                        "The configuration for the print job",
+                                                        _("Job Configuration"),
+                                                        _("The configuration for the print job"),
                                                         GNOME_TYPE_PRINT_CONFIG,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
   
@@ -273,6 +272,8 @@ gnome_print_job_get_config (GnomePrintJob *job)
  * 
  * Retrieve the GnomePrintContext which applications
  * print to.
+ *
+ * The caller is responsible to unref the context when s/he is done with it.
  * 
  * Return value: The printing context, NULL on error
  **/
@@ -283,11 +284,11 @@ gnome_print_job_get_context (GnomePrintJob *job)
 	g_return_val_if_fail (GNOME_IS_PRINT_JOB (job), NULL);
 	g_return_val_if_fail (job->input_file == NULL, NULL);
 
-	if (job->meta)
-	g_object_ref (G_OBJECT (job->meta));
-	else
+	if (job->meta == NULL)
 		job->meta = gnome_print_meta_new ();
 
+	g_object_ref (G_OBJECT (job->meta));
+
 	return job->meta;
 }
 
@@ -516,7 +517,7 @@ gnome_print_job_print (GnomePrintJob *job)
 	JOBPrivate *pp;
 	GnomePrintContext *ctx;
 	gint lpages, copies, nstacks, npages, nsheets;
-	gboolean collate, is_multipage;
+	gboolean collate, is_multipage, hwcopies;
 	gint stack;
 	const guchar *buf;
 	gint blen;
@@ -571,6 +572,15 @@ gnome_print_job_print (GnomePrintJob *job)
 	gnome_print_config_get_boolean (job->config, GNOME_PRINT_KEY_COLLATE, &collate);
 	copies = 1;
 	gnome_print_config_get_int (job->config, GNOME_PRINT_KEY_NUM_COPIES, &copies);
+	hwcopies = FALSE;
+	gnome_print_config_get_boolean (job->config, 
+					collate 
+					? GNOME_PRINT_KEY_COLLATED_COPIES_IN_HW
+					: GNOME_PRINT_KEY_NONCOLLATED_COPIES_IN_HW, 
+					&hwcopies);
+
+	if (hwcopies)
+		copies = 1;
 
 	if (collate) {
 		nstacks = copies;
@@ -601,6 +611,10 @@ gnome_print_job_print (GnomePrintJob *job)
 				}
 			}
 		}
+		if (stack + 1 < nstacks) {
+			ret = gnome_print_end_doc (ctx);
+			g_return_val_if_fail (ret == GNOME_PRINT_OK, ret);
+		}
 	}
 
 	ret = gnome_print_context_close (ctx);
@@ -865,7 +879,6 @@ job_parse_config_data (GnomePrintJob *job)
 	JOBPrivate *pp;
 	const GnomePrintUnit *unit;
 	GPANode *layout;
-	gchar *loc;
 
 	g_return_if_fail (job->priv);
 
@@ -877,8 +890,6 @@ job_parse_config_data (GnomePrintJob *job)
 
 	/* Now the fun part */
 
-	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
 
 	/* Physical size */
 	if (gnome_print_config_get_length (job->config, GNOME_PRINT_KEY_PAPER_WIDTH, &pp->pw, &unit)) {
@@ -921,7 +932,7 @@ job_parse_config_data (GnomePrintJob *job)
 						g_warning ("Could not fetch transfrom from %s\n", child_id);
 						break;
 					}
-					job_parse_transform (transform, affines + 6 * pagenum);
+					gnome_print_parse_transform (transform, affines + 6 * pagenum);
 					g_free (transform);
 					g_free (child_id);
 					pagenum += 1;
@@ -941,9 +952,6 @@ job_parse_config_data (GnomePrintJob *job)
 		art_affine_identity (pp->affines);
 		pp->num_affines = 1;
 	}
-
-	setlocale (LC_NUMERIC, loc);
-	g_free (loc);
 }
 
 #define A4_WIDTH (210 * 72 / 25.4)
@@ -982,139 +990,6 @@ job_clear_config_data (GnomePrintJob *job)
 
 /* FIXME: This function should not be here, we should not be including gpa-private.h too */
 
-/* All keys can be NULL */
-GnomePrintLayoutData *
-gnome_print_config_get_layout_data (GnomePrintConfig *config,
-				    const guchar *pagekey,
-				    const guchar *porientkey,
-				    const guchar *lorientkey,
-				    const guchar *layoutkey)
-{
-	GnomePrintLayoutData *lyd;
-	guchar key[1024];
-	const GnomePrintUnit *unit;
-	GPANode *layout;
-	gchar *loc;
-	/* Local data */
-	GnomePrintLayoutPageData *pages;
-	gdouble porient[6], lorient[6];
-	gdouble pw, ph, lyw, lyh;
-	gint num_pages;
-	gint numlp;
-	
-	g_return_val_if_fail (config != NULL, NULL);
-
-	if (!pagekey)
-		pagekey = GNOME_PRINT_KEY_PAPER_SIZE;
-	if (!porientkey)
-		porientkey = GNOME_PRINT_KEY_PAPER_ORIENTATION;
-	if (!lorientkey)
-		lorientkey = GNOME_PRINT_KEY_PAGE_ORIENTATION;
-	if (!layoutkey)
-		layoutkey = GNOME_PRINT_KEY_LAYOUT;
-
-
-	/* Initialize */
-	pw = 210 * 72.0 / 25.4;
-	ph = 297 * 72.0 / 25.4;
-	art_affine_identity (porient);
-	art_affine_identity (lorient);
-	lyw = 1.0;
-	lyh = 1.0;
-	num_pages = 0;
-	pages = NULL;
-
-	/* Now the fun part */
-
-	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-
-	/* Physical size */
-	g_snprintf (key, 1024, "%s.Width", pagekey);
-	if (gnome_print_config_get_length (config, key, &pw, &unit)) {
-		gnome_print_convert_distance (&pw, unit, GNOME_PRINT_PS_UNIT);
-	}
-	g_snprintf (key, 1024, "%s.Height", pagekey);
-	if (gnome_print_config_get_length (config, key, &ph, &unit)) {
-		gnome_print_convert_distance (&ph, unit, GNOME_PRINT_PS_UNIT);
-	}
-	/* Physical orientation */
-	g_snprintf (key, 1024, "%s.Paper2PrinterTransform", porientkey);
-	gnome_print_config_get_transform (config, key, porient);
-	/* Logical orientation */
-	g_snprintf (key, 1024, "%s.Page2LayoutTransform", lorientkey);
-	gnome_print_config_get_transform (config, key, lorient);
-	/* Layout size */
-	g_snprintf (key, 1024, "%s.Width", layoutkey);
-	gnome_print_config_get_double (config, key, &lyw);
-	g_snprintf (key, 1024, "%s.Height", layoutkey);
-	gnome_print_config_get_double (config, key, &lyh);
-
-	/* Get the layout tree from the config database */
-	layout = gpa_node_get_child_from_path (GNOME_PRINT_CONFIG_NODE (config), layoutkey);
-	if (!layout) {
-		layout = gpa_node_get_child_from_path (NULL, "Globals.Document.Page.Layout.Plain");
-		if (!layout) {
-			g_warning ("Could not get Globals.Document.Page.Layout.Plain");
-			return NULL;
-		}
-	}
-
-	/* Now come the affines */
-	numlp = 0;
-	if (gpa_node_get_int_path_value (layout, "LogicalPages", &numlp) && (numlp > 0)) {
-		GPANode *pnodes;
-		pnodes = gpa_node_get_child_from_path (layout, "Pages");
-		if (pnodes) {
-			GPANode *page;
-			gint pagenum;
-			pages = g_new (GnomePrintLayoutPageData, numlp);
-			pagenum = 0;
-			for (page = gpa_node_get_child (pnodes, NULL); page != NULL; page = gpa_node_get_child (pnodes, page)) {
-				guchar *transform;
-				transform = gpa_node_get_value (page);
-				gpa_node_unref (page);
-				if (!transform)
-					break;
-				job_parse_transform (transform, pages[pagenum].matrix);
-				g_free (transform);
-				pagenum += 1;
-				if (pagenum >= numlp)
-					break;
-			}
-			gpa_node_unref (pnodes);
-			if (pagenum == numlp) {
-				num_pages = numlp;
-			} else {
-				g_free (pages);
-			}
-		}
-	}
-	gpa_node_unref (layout);
-
-	setlocale (LC_NUMERIC, loc);
-	g_free (loc);
-	
-	if (num_pages == 0) {
-		g_warning ("Could not get_layout_data\n");
-		return NULL;
-	}
-
-	/* Success */
-
-	lyd = g_new (GnomePrintLayoutData, 1);
-	lyd->pw = pw;
-	lyd->ph = ph;
-	memcpy (lyd->porient, porient, 6 * sizeof (gdouble));
-	memcpy (lyd->lorient, lorient, 6 * sizeof (gdouble));
-	lyd->lyw = lyw;
-	lyd->lyh = lyh;
-	lyd->num_pages = num_pages;
-	lyd->pages = pages;
-
-	return lyd;
-}
-
 void
 gnome_print_layout_data_free (GnomePrintLayoutData *lyd)
 {
@@ -1245,8 +1120,8 @@ gnome_print_layout_free (GnomePrintLayout *layout)
 	g_free (layout);
 }
 
-static gboolean
-job_parse_transform (guchar *str, gdouble *transform)
+gboolean
+gnome_print_parse_transform (guchar *str, gdouble *transform)
 {
 	gdouble t[6];
 	guchar *p;
@@ -1263,7 +1138,7 @@ job_parse_transform (guchar *str, gdouble *transform)
 	if (!*p)
 		return FALSE;
 	for (i = 0; i < 6; i++) {
-		while (*p && isspace (*p)) p += 1;
+		while (*p && g_ascii_isspace (*p)) p += 1;
 		if (!strncmp (p, "SQRT2", 5)) {
 			t[i] = M_SQRT2;
 			e = p + 5;
@@ -1277,7 +1152,7 @@ job_parse_transform (guchar *str, gdouble *transform)
 			t[i] = -M_SQRT1_2;
 			e = p + 8;
 		} else {
-			t[i] = strtod (p, &e);
+			t[i] = g_ascii_strtod (p, &e);
 		}
 		if (e == (gchar *) p)
 			return FALSE;
@@ -1315,38 +1190,3 @@ gnome_print_job_print_to_file (GnomePrintJob *job, gchar *output)
 	return GNOME_PRINT_OK;
 }
 
-/* fixme: Move this (and previous) to gnome-print-config.c (Lauris) */
-
-gboolean
-gnome_print_config_get_transform (GnomePrintConfig *config, const guchar *key, gdouble *transform)
-{
-	guchar *v;
-	gdouble t[6];
-	gboolean ret;
-	gchar *loc;
-
-	g_return_val_if_fail (config != NULL, FALSE);
-	g_return_val_if_fail (key != NULL, FALSE);
-	g_return_val_if_fail (*key != '\0', FALSE);
-	g_return_val_if_fail (config != NULL, FALSE);
-
-	v = gnome_print_config_get (config, key);
-	if (!v) {
-		return FALSE;
-	}
-
-	loc = g_strdup (setlocale (LC_NUMERIC, NULL));
-	setlocale (LC_NUMERIC, "C");
-
-	ret = job_parse_transform (v, t);
-	g_free (v);
-	if (ret) {
-		memcpy (transform, t, 6 * sizeof (gdouble));
-	}
-
-	setlocale (LC_NUMERIC, loc);
-	g_free (loc);
-
-	return ret;
-}
-
