@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  gnome-print-pdf-t1.c: Tyep1 Font embeding for gnome-print-pdf
+ *  gnome-print-pdf-tt.c: TrueType Font embeding for gnome-print-pdf
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -27,34 +27,77 @@
 
 #include <config.h>
 
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <string.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-private.h>
 #include <libgnomeprint/gnome-font-private.h>
 #include <libgnomeprint/gnome-print-pdf-private.h>
+#include <libgnomeprint/ttsubset/gnome-print-tt-subset.h>
+
+#define PSO_GLYPH_MARKED(pso,g) (pso->glyphs[(g) >> 5] & (1 << ((g) & 0x1f)))
 
 gint
-gnome_print_pdf_tt_embed (GnomePrintPdf *pdf,
+gnome_print_pdf_tt_subset_embed (GnomePrintPdf *pdf,
+			  GnomePrintPdfFont *font,
 			  const guchar *file_name,
-			  gint *object_number_ret)
-{
+			  gint *object_number_ret) {
+	GnomePrintBuffer b;
 	GnomePrintReturnCode retval = GNOME_PRINT_ERROR_UNKNOWN;
-	GnomePrintBuffer b = {NULL};
-	gint object_number;
+	GnomeFontPsObject *pso = font->pso;
+	gint i, j, k, lower, upper, nglyphs, len, code_glyph;
+	guchar *subfont_file = NULL;
+	gushort glyphArray[256];
+        guchar encoding[256];
 
-	g_return_val_if_fail (file_name != NULL, retval);
+	nglyphs = pso->face->num_glyphs;
 
-	if (gnome_print_buffer_mmap (&b, file_name))
+	len = pso->encodedname ? strlen (pso->encodedname) : 0;
+
+	if (len > 4)
+		lower = *(pso->encodedname + len - 4) == '_' ? atoi (pso->encodedname + len - 3) : 0;
+	else
+		lower = 0;
+
+	upper = lower + 1;
+
+	k = 1;
+	lower *= 255;
+	upper *= 255;
+
+	font->code_to_glyph [0] = 0;
+	glyphArray[0] = encoding[0] = 0;
+
+	for ( j = lower; j < upper && j < nglyphs; j++) {
+		if (PSO_GLYPH_MARKED(pso, j)) {
+			code_glyph = j % 255 + 1;
+			glyphArray [k] = j;
+			font->code_to_glyph [code_glyph] = j;
+			encoding [k] = code_glyph;
+			k++;
+		}
+	}
+
+	for (i = 1; i <= encoding [k-1]; i++)
+		if (font->code_to_glyph [i] == -1)
+			font->code_to_glyph [i] = 0;
+
+	font->code_assigned = encoding [k-1];
+
+	(void) gnome_print_pdf_tt_create_subfont  (file_name, &subfont_file, glyphArray, encoding, k);
+
+	if (gnome_print_buffer_mmap (&b, subfont_file))
 		goto pdf_truetype_error;
 
 	if (b.buf_size < 8)
 		goto pdf_truetype_error;
 
-	object_number = gnome_print_pdf_object_new (pdf);
-	*object_number_ret = object_number;
+	*object_number_ret = gnome_print_pdf_object_new (pdf);
 	
 	/* Write the object */
-	gnome_print_pdf_object_start (pdf, object_number, FALSE);
+	gnome_print_pdf_object_start (pdf, *object_number_ret, FALSE);
 	gnome_print_pdf_fprintf    (pdf,
 				    "/Length %d" EOL
 				    "/Length1 %d" EOL
@@ -67,7 +110,7 @@ gnome_print_pdf_tt_embed (GnomePrintPdf *pdf,
 	gnome_print_pdf_fprintf (pdf,
 				 "endstream" EOL
 				 "endobj" EOL);
-	gnome_print_pdf_object_end  (pdf, object_number, TRUE);
+	gnome_print_pdf_object_end  (pdf, *object_number_ret, TRUE);
 
 	retval = GNOME_PRINT_OK;
 
@@ -75,8 +118,9 @@ pdf_truetype_error:
 	if (b.buf)
 		gnome_print_buffer_munmap (&b);
 	if (retval != GNOME_PRINT_OK)
-		g_warning ("Could not parse Type1 font from %s\n", file_name);
-	
-	return retval;
-}
+		g_warning ("Could not parse TrueType font from %s\n", subfont_file);
+	if (subfont_file)
+		unlink (subfont_file);
 
+	return retval;
+}	
