@@ -386,12 +386,13 @@ gpa_option_new_from_tree (xmlNodePtr tree, GPANode *parent)
 new_from_tree_option_loaded:
 	locked = xmlGetProp (tree, "Locked");
 	
-	if (option)
+	if (option) {
 		if (locked && !strcmp (locked, "true")) {
 			GPA_NODE_SET_FLAGS (option, NODE_FLAG_LOCKED);
 		} else {
 			GPA_NODE_UNSET_FLAGS (option, NODE_FLAG_LOCKED);
 		}		
+	}
 	
 new_from_tree_done:
 	my_xmlFree (locked);
