@@ -40,15 +40,7 @@
 #include <cups/cups.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-transport.h>
-
-/* We are protecting the locale due to a known bug in the libcups library. */
-#define LOCALEPROTECTION(c) { char *OLDLOCALE;\
-                              OLDLOCALE = g_strdup(setlocale(LC_ALL, NULL)); \
-                              c; \
-                              setlocale (LC_ALL, OLDLOCALE);\
-                              g_free (OLDLOCALE);\
-                            }
-
+#include <libgnomeprint/gnome-print-config.h>
 
 #define GP_TYPE_TRANSPORT_CUPS         (gp_transport_cups_get_type ())
 #define GP_TRANSPORT_CUPS(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GP_TYPE_TRANSPORT_CUPS, GPTransportCups))
@@ -154,7 +146,6 @@ gp_transport_cups_finalize (GObject *object)
 		transport->temp_file = NULL;
 	}
 
-	g_assert (transport->printer);
 	g_free (transport->printer);
 	transport->printer = NULL;
 
@@ -177,7 +168,9 @@ gp_transport_cups_construct (GnomePrintTransport *gp_transport)
 	}
 	
 	transport->printer = printer;
-	transport->temp_file = g_strdup ("/tmp/gnome-print-cups-XXXXXX");
+	transport->temp_file = g_build_filename(g_get_tmp_dir (), 
+						"gnome-print-cups-XXXXXX", 
+						NULL);
 
 	return GNOME_PRINT_OK;
 }
@@ -200,7 +193,7 @@ gp_transport_cups_open (GnomePrintTransport *gp_transport)
 
 	transport->file = fdopen (fd, "r+");
 
-	if (transport->file < 0) {
+	if (transport->file == NULL) {
 		g_warning ("Opening file %s for output failed", transport->temp_file);
 		return GNOME_PRINT_ERROR_UNKNOWN;
 	}
@@ -214,12 +207,19 @@ get_job_options (GnomePrintConfig *config, cups_option_t **options)
 	gchar *value;
 	gint num = 0;
 
-	value = gnome_print_config_get (config, "Settings.Output.PaperSource");
+	value = gnome_print_config_get (config, GNOME_PRINT_KEY_PAPER_SOURCE);
 
-	LOCALEPROTECTION (num = cupsAddOption 
-			  ("InputSlot", value, num, options))
+	if (value != NULL) {
+		num = cupsAddOption ("InputSlot", value, num, options);
+		g_free (value);
+	}
+	
+	value = gnome_print_config_get (config, GNOME_PRINT_KEY_HOLD);
 
-	g_free (value);
+	if (value != NULL) {
+		num = cupsAddOption ("job-hold-until", value, num, options);
+		g_free (value);
+	}
 	
 	return num;
 }
@@ -235,7 +235,8 @@ gp_transport_cups_close (GnomePrintTransport *gp_transport)
 
 	transport = GP_TRANSPORT_CUPS (gp_transport);
 
-	g_return_val_if_fail (transport->file >= 0, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (transport->file != NULL,
+			      GNOME_PRINT_ERROR_UNKNOWN);
 
 	if (fclose (transport->file) < 0) {
 		g_warning ("Closing output file failed");
@@ -247,10 +248,10 @@ gp_transport_cups_close (GnomePrintTransport *gp_transport)
 					GNOME_PRINT_KEY_DOCUMENT_NAME);
 	num_options = get_job_options (gp_transport->config, &options);
 	
-	LOCALEPROTECTION (cupsPrintFile (transport->printer, 
-					 transport->temp_file, 
-					 title, num_options, options);
-			  cupsFreeOptions (num_options, options))
+	cupsPrintFile (transport->printer, 
+		       transport->temp_file, 
+		       title, num_options, options);
+	cupsFreeOptions (num_options, options);
 
 	unlink (transport->temp_file);
 	g_free (title);
@@ -266,7 +267,8 @@ gp_transport_cups_write (GnomePrintTransport *gp_transport, const guchar *buf, g
 
 	transport = GP_TRANSPORT_CUPS (gp_transport);
 
-	g_return_val_if_fail (transport->file >= 0, GNOME_PRINT_ERROR_UNKNOWN);
+	g_return_val_if_fail (transport->file != NULL,
+			      GNOME_PRINT_ERROR_UNKNOWN);
 
 	l = len;
 	while (l > 0) {
@@ -298,10 +300,10 @@ gp_transport_cups_print_file (GnomePrintTransport *gp_transport, const guchar *f
 					GNOME_PRINT_KEY_DOCUMENT_NAME);
 	num_options = get_job_options (gp_transport->config, &options);
 	
-	LOCALEPROTECTION(cupsPrintFile (transport->printer, 
-					filename, title, 
-					num_options, options);
-			 cupsFreeOptions (num_options, options))
+	cupsPrintFile (transport->printer, 
+		       filename, title, 
+		       num_options, options);
+	cupsFreeOptions (num_options, options);
 
 	g_free (title);
 	
