@@ -33,10 +33,10 @@
  *
  */
 
-/* $Id: sft.h,v 1.3 2004/07/20 22:04:39 jody Exp $ */
+/* $Id: sft.h,v 1.5 2004/12/03 18:32:57 jody Exp $ */
 /* @(#)sft.h 1.17 03/01/08 SMI */
 
-/**
+/*
  * @file sft.h
  * @brief Sun Font Tools
  * @author Alexander Gelfenbain <adg@sun.com>
@@ -77,7 +77,9 @@
 #define __SUBFONT_H
 
 #include <sys/types.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <stdio.h>
 #ifndef NO_LIST
 #include "list.h"
@@ -150,6 +152,21 @@ extern "C" {
 /*@}*/
 
 #ifndef STSF
+#if 1
+/* glib already deals with different compilers */
+#include <glibconfig.h>
+
+typedef guchar     byte;
+typedef guchar     uint8;
+typedef gchar      int8;
+typedef guint16    uint16;
+typedef gint16     int16;
+
+typedef guint32    uint32;
+typedef gint32     int32;
+typedef guint64    uint64;
+typedef gint64     int64;
+#  else
 /*- XXX These should be dynamically configured */ /*FOLD00*/
     typedef unsigned char         byte;
     typedef unsigned char         uint8;
@@ -161,7 +178,7 @@ extern "C" {
     typedef int                   int32;
     typedef unsigned long long    uint64;
     typedef long long             int64;
-
+#  endif
 /*@{*/
     typedef int16       F2Dot14;            /**< fixed: 2.14 */
     typedef int32       F16Dot16;           /**< fixed: 16.16 */
@@ -377,12 +394,12 @@ extern "C" {
 
     typedef struct _TrueTypeFont TrueTypeFont;
 
-/**
+/*
  * @defgroup sft Sun Font Tools Exported Functions
  */
 
 
-/**
+/*
  * Get the number of fonts contained in a TrueType collection
  * @param  fname - file name
  * @return number of fonts or zero, if file is not a TTC file.
@@ -391,7 +408,7 @@ extern "C" {
     int CountTTCFonts(const char* fname);
 
 
-/**
+/*
  * TrueTypeFont constructor. 
  * Reads the font file and allocates the memory for the structure.
  * @param  facenum - logical font number within a TTC file. This value is ignored
@@ -401,13 +418,13 @@ extern "C" {
  */
     int  OpenTTFont(const char *fname, uint32 facenum, TrueTypeFont**);
 
-/**
+/*
  * TrueTypeFont destructor. Deallocates the memory.
  * @ingroup sft
  */
     void CloseTTFont(TrueTypeFont *);
 
-/**
+/*
  * Extracts TrueType control points, and stores them in an allocated array pointed to
  * by *pointArray. This function returns the number of extracted points.
  *
@@ -420,22 +437,22 @@ extern "C" {
  * @ingroup sft
  *
  */
-    int GetTTGlyphPoints(TrueTypeFont *ttf, uint32 glyphID, ControlPoint **pointArray);
+int GetTTGlyphPoints(TrueTypeFont *ttf, uint32 glyphID, ControlPoint **pointArray);
 
-    /**
-     * Extracts bounding boxes in normalized FUnits (1000/em) for all glyphs in the
-     * font and allocates an array of FUnitBBox structures. There are ttf->nglyphs
-     * elements in the array.
-     *
-     * @param ttf     pointer to the TrueTypeFont structure
-     *
-     * @return        an array of FUnitBBox structures with values normalized to 1000 UPEm
-     *
-     * @ingroup sft
-     */
-    FUnitBBox *GetTTGlyphBoundingBoxes(TrueTypeFont *ttf);
+/*
+ * Extracts bounding boxes in normalized FUnits (1000/em) for all glyphs in the
+ * font and allocates an array of FUnitBBox structures. There are ttf->nglyphs
+ * elements in the array.
+ *
+ * @param ttf     pointer to the TrueTypeFont structure
+ *
+ * @return        an array of FUnitBBox structures with values normalized to 1000 UPEm
+ *
+ * @ingroup sft
+ */
+FUnitBBox *GetTTGlyphBoundingBoxes(TrueTypeFont *ttf);
 
-/**
+/*
  * Extracts raw glyph data from the 'glyf' table and returns it in an allocated
  * GlyphData structure.
  *
@@ -450,7 +467,7 @@ extern "C" {
     GlyphData *GetTTRawGlyphData(TrueTypeFont *ttf, uint32 glyphID);
 
 #ifndef NO_LIST
-/**
+/*
  * For a specified glyph adds all component glyphs IDs to the list and
  * return their number. If the glyph is a single glyph it has one component
  * glyph (which is added to the list) and the function returns 1.
@@ -468,7 +485,7 @@ extern "C" {
     int GetTTGlyphComponents(TrueTypeFont *ttf, uint32 glyphID, list glyphlist);
 #endif
 
-/**
+/*
  * Extracts all Name Records from the font and stores them in an allocated
  * array of NameRecord structs
  *
@@ -481,7 +498,7 @@ extern "C" {
 
     int GetTTNameRecords(TrueTypeFont *ttf, NameRecord **nr);
 
-/**
+/*
  * Deallocates previously allocated array of NameRecords.
  *
  * @param nr        array of NameRecord structs
@@ -493,7 +510,7 @@ extern "C" {
 
 
 #ifndef NO_TYPE3
-/**
+/*
  * Generates a new PostScript Type 3 font and dumps it to <b>outf</b> file.
  * This functions subsititues glyph 0 for all glyphIDs that are not found in the font.
  * @param ttf         pointer to the TrueTypeFont structure
@@ -514,7 +531,7 @@ extern "C" {
 #endif
 
 #ifndef NO_TTCR
-/**
+/*
  * Generates a new TrueType font and dumps it to <b>outf</b> file.
  * This functions subsititues glyph 0 for all glyphIDs that are not found in the font.
  * @param ttf         pointer to the TrueTypeFont structure
@@ -545,7 +562,7 @@ extern "C" {
 #endif
 
 #ifndef NO_TYPE42
-/**
+/*
  * Generates a new PostScript Type42 font and dumps it to <b>outf</b> file.
  * This functions subsititues glyph 0 for all glyphIDs that are not found in the font.
  * @param ttf         pointer to the TrueTypeFont structure
@@ -573,26 +590,26 @@ extern "C" {
                                int            nGlyphs);
 #endif
 
-    /**
-     * Queries full glyph metrics for one glyph
-     */
-    void GetTTGlyphMetrics(TrueTypeFont *ttf, uint32 glyphID, TTGlyphMetrics *metrics);
+/*
+ * Queries full glyph metrics for one glyph
+ */
+void GetTTGlyphMetrics(TrueTypeFont *ttf, uint32 glyphID, TTGlyphMetrics *metrics);
 
 
-    /**
-     * Queries glyph metrics. Allocates an array of TTSimpleGlyphMetrics structs and returns it.
-     *
-     * @param ttf         pointer to the TrueTypeFont structure
-     * @param glyphArray  pointer to an array of glyphs that are to be extracted from ttf
-     * @param nGlyphs     number of glyph IDs in glyphArray and encoding values in encoding
-     * @param mode        writing mode: 0 - horizontal, 1 - vertical
-     * @ingroup sft
-     *
-     */
-    TTSimpleGlyphMetrics *GetTTSimpleGlyphMetrics(TrueTypeFont *ttf, uint16 *glyphArray, int nGlyphs, int mode);
+/*
+ * Queries glyph metrics. Allocates an array of TTSimpleGlyphMetrics structs and returns it.
+ *
+ * @param ttf         pointer to the TrueTypeFont structure
+ * @param glyphArray  pointer to an array of glyphs that are to be extracted from ttf
+ * @param nGlyphs     number of glyph IDs in glyphArray and encoding values in encoding
+ * @param mode        writing mode: 0 - horizontal, 1 - vertical
+ * @ingroup sft
+ *
+ */
+TTSimpleGlyphMetrics *GetTTSimpleGlyphMetrics(TrueTypeFont *ttf, uint16 *glyphArray, int nGlyphs, int mode);
 
 #ifndef NO_MAPPERS
-/**
+/*
  * Queries character metrics. Allocates an array of TTSimpleGlyphMetrics structs and returns it.
  * This function behaves just like GetTTSimpleGlyphMetrics() but it takes a range of Unicode
  * characters instead of an array of glyphs.
@@ -608,7 +625,7 @@ extern "C" {
  */
     TTSimpleGlyphMetrics *GetTTSimpleCharMetrics(TrueTypeFont *ttf, uint16 firstChar, int nChars, int mode);
 
-/**
+/*
  * Maps a Unicode (UCS-2) string to a glyph array. Returns the number of glyphs in the array,
  * which for TrueType fonts is always the same as the number of input characters.
  *
@@ -632,7 +649,7 @@ extern "C" {
     int MapString(TrueTypeFont *ttf, uint16 *str, int nchars, uint16 *glyphArray);
 #endif
 
-/**
+/*
  * Maps a Unicode (UCS-2) character to a glyph ID and returns it. Missing glyph has
  * a glyphID of 0 so this function can be used to test if a character is encoded in the font.
  *
@@ -648,7 +665,7 @@ extern "C" {
 #endif
 #endif
 
-/**
+/*
  * Returns global font information about the TrueType font.
  * @see TTGlobalFontInfo
  *
@@ -659,7 +676,7 @@ extern "C" {
  */
     void GetTTGlobalFontInfo(TrueTypeFont *ttf, TTGlobalFontInfo *info);
 
-/**
+/*
  * Returns kerning information for an array of glyphs.
  * Kerning is not cumulative.
  * kern[i] contains kerning information for a pair of glyphs at positions i and i+1
@@ -673,117 +690,117 @@ extern "C" {
  * @ingroup sft
  *
  */
-    void KernGlyphs(TrueTypeFont *ttf, uint16 *glyphs, int nglyphs, int wmode, KernData *kern);
+void KernGlyphs(TrueTypeFont *ttf, uint16 *glyphs, int nglyphs, int wmode, KernData *kern);
 
-    /**
-     * Returns nonzero if font is a symbol encoded font
-     */
-    int CheckSymbolEncoding(TrueTypeFont* ttf);
+/*
+ * Returns nonzero if font is a symbol encoded font
+ */
+int CheckSymbolEncoding(TrueTypeFont* ttf);
 
-    /**
-     * Extracts a 'cmap' table from a font, allocates memory for it and returns a pointer to it.
-     * DEPRECATED - use ExtractTable instead
-     */
+/*
+ * Extracts a 'cmap' table from a font, allocates memory for it and returns a pointer to it.
+ * DEPRECATED - use ExtractTable instead
+ */
 #if 0
-    byte *ExtractCmap(TrueTypeFont *ttf);
+byte *ExtractCmap(TrueTypeFont *ttf);
 #endif
 
-    /**
-     * Extracts a table from a font, allocates memort for it and returns a pointer to it.
-     */
-    byte *ExtractTable(TrueTypeFont *ttf, uint32 tag);
+/*
+ * Extracts a table from a font, allocates memort for it and returns a pointer to it.
+ */
+byte *ExtractTable(TrueTypeFont *ttf, uint32 tag);
 
-    /**
-     * Returns a pointer to the table but does not allocate memory for it.
-     */
-    const byte *GetTable(TrueTypeFont *ttf, uint32 tag);
+/*
+ * Returns a pointer to the table but does not allocate memory for it.
+ */
+const byte *GetTable(TrueTypeFont *ttf, uint32 tag);
 
-    /*
-     * Functions that do not use TrueTypeFont structure
-     * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-     */
+/*
+ * Functions that do not use TrueTypeFont structure
+ * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+ */
 
-    /**
-     * Reads full (vertical and horisontal) glyph metrics for an array of glyphs from hmtx and vmtx tables
-     *
-     * @param hmtx       TrueType hmtx table
-     * @param vmtx       TrueType vmtx table
-     * @param hcount     numberOfHMetrics value
-     * @param vcount     numOfLongVerMetrics value
-     * @param gcount     total number of glyphs in the font
-     * @param UPEm       units per Em value
-     * @param glyphArray array of source glyph IDs
-     * @param nGlyphs    number of glyphs in the glyphArray array
-     *
-     * @return           array of TTFullSimpleGlyphMetrics data structures
-     *
-     */
+/*
+ * Reads full (vertical and horisontal) glyph metrics for an array of glyphs from hmtx and vmtx tables
+ *
+ * @param hmtx       TrueType hmtx table
+ * @param vmtx       TrueType vmtx table
+ * @param hcount     numberOfHMetrics value
+ * @param vcount     numOfLongVerMetrics value
+ * @param gcount     total number of glyphs in the font
+ * @param UPEm       units per Em value
+ * @param glyphArray array of source glyph IDs
+ * @param nGlyphs    number of glyphs in the glyphArray array
+ *
+ * @return           array of TTFullSimpleGlyphMetrics data structures
+ *
+ */
 
-    TTFullSimpleGlyphMetrics *ReadGlyphMetrics(byte *hmtx, byte *vmtx, int hcount, int vcount, int gcount, int UPEm, uint16 *glyphArray, int nGlyphs);
+TTFullSimpleGlyphMetrics *ReadGlyphMetrics(byte *hmtx, byte *vmtx, int hcount, int vcount, int gcount, int UPEm, uint16 *glyphArray, int nGlyphs);
 
-    void ReadSingleGlyphMetrics(byte *hmtx, byte *vmtx, int hcount, int vcount, int gcount, int UPEm, uint16 glyphID, TTFullSimpleGlyphMetrics *metrics);
-    
+void ReadSingleGlyphMetrics(byte *hmtx, byte *vmtx, int hcount, int vcount, int gcount, int UPEm, uint16 glyphID, TTFullSimpleGlyphMetrics *metrics);
+
+
+/*
+ * Returns the length of the 'kern' subtable
+ *
+ * @param kern       pointer to the 'kern' subtable
+ *
+ * @return           number of bytes in it
+ */
+uint32 GetKernSubtableLength(byte *kern);
+
+/*
+ *  Kerns a pair of glyphs.
+ *
+ * @param kerntype   type of the kern table
+ * @param nkern      number of kern subtables
+ * @param kern       array of pointers to kern subtables
+ * @pram unitsPerEm  units per Em value
+ * @param wmode      writing mode: 0 - horizontal, 1 - vertical
+ * @param a          ID of the first glyoh
+ * @param b          ID of the second glyoh
+ * @param x          X-axis kerning value is returned here
+ * @param y          Y-axis kerning value is returned here
+ */
+void KernGlyphPair(int kerntype, uint32 nkern, byte **kern, int unitsPerEm, int wmode, uint32 a, uint32 b, int *x, int *y);
 
-    /*
-     * Returns the length of the 'kern' subtable
-     *
-     * @param kern       pointer to the 'kern' subtable
-     *
-     * @return           number of bytes in it
-     */
-    uint32 GetKernSubtableLength(byte *kern);
 
-    /*
-     *  Kerns a pair of glyphs.
-     *
-     * @param kerntype   type of the kern table
-     * @param nkern      number of kern subtables
-     * @param kern       array of pointers to kern subtables
-     * @pram unitsPerEm  units per Em value
-     * @param wmode      writing mode: 0 - horizontal, 1 - vertical
-     * @param a          ID of the first glyoh
-     * @param b          ID of the second glyoh
-     * @param x          X-axis kerning value is returned here
-     * @param y          Y-axis kerning value is returned here
-     */
-    void KernGlyphPair(int kerntype, uint32 nkern, byte **kern, int unitsPerEm, int wmode, uint32 a, uint32 b, int *x, int *y);
 
-    
-    
 /*- private definitions */ /*FOLD00*/
 
-    struct _TrueTypeFont {
-        uint32 tag;
-    
-        char   *fname;
-        off_t  fsize;
-        byte   *ptr;
+struct _TrueTypeFont {
+    uint32 tag;
 
-        char   *psname;
-        char   *family;
-        uint16  *ufamily;
-        char   *subfamily;
+    char   *fname;
+    off_t  fsize;
+    byte   *ptr;
 
-        uint32 ntables;
-        uint32 tdoffset;                              /* offset to the table directory (!= 0 for TrueType collections)      */
-        uint32 *goffsets;
-        int    nglyphs;
-        int    unitsPerEm;
-        int    numberOfHMetrics;
-        int    numOfLongVerMetrics;                   /* if this number is not 0, font has vertical metrics information */
-        byte   *cmap;
-        int    cmapType;
-        uint16 (*mapper)(const byte *, uint16);       /* character to glyphID translation function                          */
-        void   **tables;                              /* array of pointers to tables                                        */
-        uint32 *tlens;                                /* array of table lengths                                             */
-        int    kerntype;                              /* Defined in the KernType enum                                       */
-        uint32 nkern;                                 /* number of kern subtables                                           */
-        byte   **kerntables;                          /* array of pointers to kern subtables                                */
+    char   *psname;
+    char   *family;
+    uint16  *ufamily;
+    char   *subfamily;
+
+    uint32 ntables;
+    uint32 tdoffset;                              /* offset to the table directory (!= 0 for TrueType collections)      */
+    uint32 *goffsets;
+    int    nglyphs;
+    int    unitsPerEm;
+    int    numberOfHMetrics;
+    int    numOfLongVerMetrics;                   /* if this number is not 0, font has vertical metrics information */
+    byte   *cmap;
+    int    cmapType;
+    uint16 (*mapper)(const byte *, uint16);       /* character to glyphID translation function                          */
+    void   **tables;                              /* array of pointers to tables                                        */
+    uint32 *tlens;                                /* array of table lengths                                             */
+    int    kerntype;                              /* Defined in the KernType enum                                       */
+    uint32 nkern;                                 /* number of kern subtables                                           */
+    byte   **kerntables;                          /* array of pointers to kern subtables                                */
 #ifdef USE_GSUB
-        void   *pGSubstitution;                       /* info provided by GSUB for UseGSUB()                                */
+    void   *pGSubstitution;                       /* info provided by GSUB for UseGSUB()                                */
 #endif
-	GnomePrintBuffer gp_buf;
-    };
+    GnomePrintBuffer gp_buf;
+};
 
 #ifdef __cplusplus
 }
